@@ -7,9 +7,9 @@
 
    see qh-geom.htm and geom.h
 
-   Copyright (c) 1993-2018 The Geometry Center.
-   $Id: //main/2015/qhull/src/libqhull/geom2.c#9 $$Change: 2549 $
-   $DateTime: 2018/12/28 22:24:20 $$Author: bbarber $
+   Copyright (c) 1993-2019 The Geometry Center.
+   $Id: //main/2019/qhull/src/libqhull/geom2.c#11 $$Change: 2712 $
+   $DateTime: 2019/06/28 12:57:00 $$Author: bbarber $
 
    frequently used code goes into geom.c
 */
@@ -21,8 +21,9 @@
 /*-<a                             href="qh-geom.htm#TOC"
   >-------------------------------</a><a name="copypoints">-</a>
 
-  qh_copypoints( points, numpoints, dimension)
+  qh_copypoints( points, numpoints, dimension )
     return qh_malloc'd copy of points
+
   notes:
     qh_free the returned points to avoid a memory leak
 */
@@ -31,7 +32,7 @@ coordT *qh_copypoints(coordT *points, int numpoints, int dimension) {
   coordT *newpoints;
 
   size= numpoints * dimension * (int)sizeof(coordT);
-  if (!(newpoints=(coordT*)qh_malloc((size_t)size))) {
+  if (!(newpoints= (coordT *)qh_malloc((size_t)size))) {
     qh_fprintf(qh ferr, 6004, "qhull error: insufficient memory to copy %d points\n",
         numpoints);
     qh_errexit(qh_ERRmem, NULL, NULL);
@@ -91,16 +92,16 @@ realT qh_determinant(realT **rows, int dim, boolT *nearzero) {
   }else if (dim == 2) {
     det= det2_(rows[0][0], rows[0][1],
                  rows[1][0], rows[1][1]);
-    if (fabs_(det) < 10*qh NEARzero[1])  /* not really correct, what should this be? */
+    if (fabs_(det) < 10*qh NEARzero[1])  /* QH11031 FIX: not really correct, what should this be? */
       *nearzero= True;
   }else if (dim == 3) {
     det= det3_(rows[0][0], rows[0][1], rows[0][2],
                  rows[1][0], rows[1][1], rows[1][2],
                  rows[2][0], rows[2][1], rows[2][2]);
-    if (fabs_(det) < 10*qh NEARzero[2])  /* what should this be?  det 5.5e-12 was flat for qh_maxsimplex of qdelaunay 0,0 27,27 -36,36 -9,63  */
+    if (fabs_(det) < 10*qh NEARzero[2])  /* QH11031 FIX: what should this be?  det 5.5e-12 was flat for qh_maxsimplex of qdelaunay 0,0 27,27 -36,36 -9,63 */
       *nearzero= True;
   }else {
-    qh_gausselim(rows, dim, dim, &sign, nearzero);  /* if nearzero, diagonal still ok*/
+    qh_gausselim(rows, dim, dim, &sign, nearzero);  /* if nearzero, diagonal still ok */
     det= 1.0;
     for (i=dim; i--; )
       det *= (rows[i])[i];
@@ -134,35 +135,62 @@ realT qh_detjoggle(pointT *points, int numpoints, int dimension) {
   realT maxwidth= 0;
   int k;
 
-  for (k=0; k < dimension; k++) {
-    if (qh SCALElast && k == dimension-1)
-      abscoord= maxwidth;
-    else if (qh DELAUNAY && k == dimension-1) /* will qh_setdelaunay() */
-      abscoord= 2 * maxabs * maxabs;  /* may be low by qh hull_dim/2 */
-    else {
-      maxcoord= -REALmax;
-      mincoord= REALmax;
-      FORALLpoint_(points, numpoints) {
-        maximize_(maxcoord, point[k]);
-        minimize_(mincoord, point[k]);
+  if (qh SETroundoff)
+    distround= qh DISTround; /* 'En' */
+  else{
+    for (k=0; k < dimension; k++) {
+      if (qh SCALElast && k == dimension-1)
+        abscoord= maxwidth;
+      else if (qh DELAUNAY && k == dimension-1) /* will qh_setdelaunay() */
+        abscoord= 2 * maxabs * maxabs;  /* may be low by qh hull_dim/2 */
+      else {
+        maxcoord= -REALmax;
+        mincoord= REALmax;
+        FORALLpoint_(points, numpoints) {
+          maximize_(maxcoord, point[k]);
+          minimize_(mincoord, point[k]);
+        }
+        maximize_(maxwidth, maxcoord-mincoord);
+        abscoord= fmax_(maxcoord, -mincoord);
       }
-      maximize_(maxwidth, maxcoord-mincoord);
-      abscoord= fmax_(maxcoord, -mincoord);
-    }
-    sumabs += abscoord;
-    maximize_(maxabs, abscoord);
-  } /* for k */
-  distround= qh_distround(qh hull_dim, maxabs, sumabs);
+      sumabs += abscoord;
+      maximize_(maxabs, abscoord);
+    } /* for k */
+    distround= qh_distround(qh hull_dim, maxabs, sumabs);
+  }
   joggle= distround * qh_JOGGLEdefault;
   maximize_(joggle, REALepsilon * qh_JOGGLEdefault);
   trace2((qh ferr, 2001, "qh_detjoggle: joggle=%2.2g maxwidth=%2.2g\n", joggle, maxwidth));
   return joggle;
 } /* detjoggle */
 
+/*-<a                             href="qh-geom.htm#TOC"
+  >-------------------------------</a><a name="detmaxoutside">-</a>
+
+  qh_detmaxoutside();
+    determine qh.MAXoutside target for qh_RATIO... tests of distance
+    updates option '_max-outside'
+
+  notes:
+    called from qh_addpoint and qh_detroundoff
+    accounts for qh.ONEmerge, qh.DISTround, qh.MINoutside ('Wn'), qh.max_outside
+    see qh_maxout for qh.max_outside with qh.DISTround
+*/
+
+void qh_detmaxoutside(void) {
+  realT maxoutside;
+
+  maxoutside= fmax_(qh max_outside, qh ONEmerge + qh DISTround);
+  maximize_(maxoutside, qh MINoutside);
+  qh MAXoutside= maxoutside;
+  trace3((qh ferr, 3056, "qh_detmaxoutside: MAXoutside %2.2g from qh.max_outside %2.2g, ONEmerge %2.2g, MINoutside %2.2g, DISTround %2.2g\n",
+      qh MAXoutside, qh max_outside, qh ONEmerge, qh MINoutside, qh DISTround));
+} /* detmaxoutside */
+
 /*-<a                             href="qh-geom.htm#TOC"
   >-------------------------------</a><a name="detroundoff">-</a>
 
-  qh_detroundoff()
+  qh_detroundoff( )
     determine maximum roundoff errors from
       REALepsilon, REALmax, REALmin, qh.hull_dim, qh.MAXabs_coord,
       qh.MAXsumcoord, qh.MAXwidth, qh.MINdenom_1
@@ -194,8 +222,6 @@ void qh_detroundoff(void) {
   qh_option("_max-width", NULL, &qh MAXwidth);
   if (!qh SETroundoff) {
     qh DISTround= qh_distround(qh hull_dim, qh MAXabs_coord, qh MAXsumcoord);
-    if (qh RANDOMdist)
-      qh DISTround += qh RANDOMfactor * qh MAXabs_coord;
     qh_option("Error-roundoff", NULL, &qh DISTround);
   }
   qh MINdenom= qh MINdenom_1 * qh MAXabs_coord;
@@ -203,8 +229,10 @@ void qh_detroundoff(void) {
   qh MINdenom_2= qh MINdenom_1_2 * qh MAXabs_coord;
                                               /* for inner product */
   qh ANGLEround= 1.01 * qh hull_dim * REALepsilon;
-  if (qh RANDOMdist)
+  if (qh RANDOMdist) {
     qh ANGLEround += qh RANDOMfactor;
+    trace4((qh ferr, 4096, "qh_detroundoff: increase qh.ANGLEround by option 'R%2.2g'\n", qh RANDOMfactor));
+  }
   if (qh premerge_cos < REALmax/2) {
     qh premerge_cos -= qh ANGLEround;
     if (qh RANDOMdist)
@@ -247,7 +275,7 @@ void qh_detroundoff(void) {
   if (qh KEEPnearinside)
     qh_option("_near-inside", NULL, &qh NEARinside);
   if (qh JOGGLEmax < qh DISTround) {
-    qh_fprintf(qh ferr, 6006, "qhull error: the joggle for 'QJn', %.2g, is below roundoff for distance computations, %.2g\n",
+    qh_fprintf(qh ferr, 6006, "qhull option error: the joggle for 'QJn', %.2g, is below roundoff for distance computations, %.2g\n",
          qh JOGGLEmax, qh DISTround);
     qh_errexit(qh_ERRinput, NULL, NULL);
   }
@@ -264,7 +292,7 @@ void qh_detroundoff(void) {
   }
   if (qh MAXcoplanar > REALmax/2) {
     qh MAXcoplanar= qh MINvisible;
-    qh_option("U-coplanar-distance", NULL, &qh MAXcoplanar);
+    qh_option("U-max-coplanar", NULL, &qh MAXcoplanar);
   }
   if (!qh APPROXhull) {             /* user may specify qh MINoutside */
     qh MINoutside= 2 * qh MINvisible;
@@ -283,6 +311,7 @@ void qh_detroundoff(void) {
   qh max_vertex= qh DISTround;
   qh min_vertex= -qh DISTround;
   /* numeric constants reported in printsummary */
+  qh_detmaxoutside();
 } /* detroundoff */
 
 /*-<a                             href="qh-geom.htm#TOC"
@@ -295,6 +324,7 @@ void qh_detroundoff(void) {
      signed determinant and nearzero from qh_determinant
 
   notes:
+     called by qh_maxsimplex and qh_initialvertices
      uses qh.gm_matrix/qh.gm_row (assumes they're big enough)
 
   design:
@@ -359,14 +389,15 @@ realT qh_distnorm(int dim, pointT *point, pointT *normal, realT *offsetp) {
 /*-<a                             href="qh-geom.htm#TOC"
   >-------------------------------</a><a name="distround">-</a>
 
-  qh_distround(dimension, maxabs, maxsumabs )
+  qh_distround( dimension, maxabs, maxsumabs )
     compute maximum round-off error for a distance computation
       to a normalized hyperplane
     maxabs is the maximum absolute value of a coordinate
     maxsumabs is the maximum possible sum of absolute coordinate values
+    if qh.RANDOMdist ('Qr'), adjusts qh_distround
 
   returns:
-    max dist round for REALepsilon
+    max dist round for qh.REALepsilon and qh.RANDOMdist
 
   notes:
     calculate roundoff error according to Golub & van Loan, 1983, Lemma 3.2-1, "Rounding Errors"
@@ -374,14 +405,19 @@ realT qh_distnorm(int dim, pointT *point, pointT *normal, realT *offsetp) {
       or use maxsumabs since one vector is < 1
 */
 realT qh_distround(int dimension, realT maxabs, realT maxsumabs) {
-  realT maxdistsum, maxround;
+  realT maxdistsum, maxround, delta;
 
   maxdistsum= sqrt((realT)dimension) * maxabs;
   minimize_( maxdistsum, maxsumabs);
   maxround= REALepsilon * (dimension * maxdistsum * 1.01 + maxabs);
               /* adds maxabs for offset */
-  trace4((qh ferr, 4008, "qh_distround: %2.2g maxabs %2.2g maxsumabs %2.2g maxdistsum %2.2g\n",
-                 maxround, maxabs, maxsumabs, maxdistsum));
+  if (qh RANDOMdist) {
+    delta= qh RANDOMfactor * maxabs;
+    maxround += delta;
+    trace4((qh ferr, 4092, "qh_distround: increase roundoff by random delta %2.2g for option 'R%2.2g'\n", delta, qh RANDOMfactor));
+  }
+  trace4((qh ferr, 4008, "qh_distround: %2.2g, maxabs %2.2g, maxsumabs %2.2g, maxdistsum %2.2g\n",
+            maxround, maxabs, maxsumabs, maxdistsum));
   return maxround;
 } /* distround */
 
@@ -566,7 +602,7 @@ realT qh_facetarea_simplex(int dim, coordT *apex, setT *vertices,
     for (k=dim; k--; )
       *(gmcoord++)= *normalp++;
   }
-  zinc_(Zdetsimplex);
+  zinc_(Zdetfacetarea);
   area= qh_determinant(rows, dim, &nearzero);
   if (toporient)
     area= -area;
@@ -676,6 +712,90 @@ facetT *qh_findgooddist(pointT *point, facetT *facetA, realT *distp,
   return NULL;
 }  /* findgooddist */
 
+/*-<a                             href="qh-geom.htm#TOC"
+  >-------------------------------</a><a name="furthestnewvertex">-</a>
+
+  qh_furthestnewvertex( unvisited, facet, &maxdist )
+    return furthest unvisited, new vertex to a facet
+
+  return:
+    NULL if no vertex is above facet
+    maxdist to facet
+    updates v.visitid
+
+  notes:
+    Ignores vertices in facetB
+    Does not change qh.vertex_visit.  Use in conjunction with qh_furthestvertex
+*/
+vertexT *qh_furthestnewvertex(unsigned int unvisited, facetT *facet, realT *maxdistp /* qh.newvertex_list */) {
+  vertexT *maxvertex= NULL, *vertex;
+  coordT dist, maxdist= 0.0;
+
+  FORALLvertex_(qh newvertex_list) {
+    if (vertex->newfacet && vertex->visitid <= unvisited) {
+      vertex->visitid= qh vertex_visit;
+      qh_distplane(vertex->point, facet, &dist);
+      if (dist > maxdist) {
+        maxdist= dist;
+        maxvertex= vertex;
+      }
+    }
+  }
+  trace4((qh ferr, 4085, "qh_furthestnewvertex: v%d dist %2.2g is furthest new vertex for f%d\n",
+    getid_(maxvertex), maxdist, facet->id));
+  *maxdistp= maxdist;
+  return maxvertex;
+} /* furthestnewvertex */
+
+/*-<a                             href="qh-geom.htm#TOC"
+  >-------------------------------</a><a name="furthestvertex">-</a>
+
+  qh_furthestvertex( facetA, facetB, &maxdist, &mindist )
+    return furthest vertex in facetA from facetB, or NULL if none
+
+  return:
+    maxdist and mindist to facetB or 0.0 if none
+    updates qh.vertex_visit
+
+  notes:
+    Ignores vertices in facetB
+*/
+vertexT *qh_furthestvertex(facetT *facetA, facetT *facetB, realT *maxdistp, realT *mindistp) {
+  vertexT *maxvertex= NULL, *vertex, **vertexp;
+  coordT dist, maxdist= -REALmax, mindist= REALmax;
+
+  qh vertex_visit++;
+  FOREACHvertex_(facetB->vertices)
+    vertex->visitid= qh vertex_visit;
+  FOREACHvertex_(facetA->vertices) {
+    if (vertex->visitid != qh vertex_visit) {
+      vertex->visitid= qh vertex_visit;
+      zzinc_(Zvertextests);
+      qh_distplane(vertex->point, facetB, &dist);
+      if (!maxvertex) {
+        maxdist= dist;
+        mindist= dist;
+        maxvertex= vertex;
+      }else if (dist > maxdist) {
+        maxdist= dist;
+        maxvertex= vertex;
+      }else if (dist < mindist)
+        mindist= dist;
+    }
+  }
+  if (!maxvertex) {
+    trace3((qh ferr, 3067, "qh_furthestvertex: all vertices of f%d are in f%d.  Returning 0.0 for max and mindist\n",
+      facetA->id, facetB->id));
+    maxdist= mindist= 0.0;
+  }else {
+    trace4((qh ferr, 4084, "qh_furthestvertex: v%d dist %2.2g is furthest (mindist %2.2g) of f%d above f%d\n",
+      maxvertex->id, maxdist, mindist, facetA->id, facetB->id));
+  }
+  *maxdistp= maxdist;
+  *mindistp= mindist;
+  return maxvertex;
+} /* furthestvertex */
+
 /*-<a                             href="qh-geom.htm#TOC"
   >-------------------------------</a><a name="getarea">-</a>
 
@@ -709,7 +829,7 @@ void qh_getarea(facetT *facetlist) {
   if (qh REPORTfreq)
     qh_fprintf(qh ferr, 8020, "computing area of each facet and volume of the convex hull\n");
   else
-    trace1((qh ferr, 1001, "qh_getarea: computing volume and area for each facet\n"));
+    trace1((qh ferr, 1001, "qh_getarea: computing area for each facet and its volume to qh.interior_point (dist*area/dim)\n"));
   qh totarea= qh totvol= 0.0;
   FORALLfacet_(facetlist) {
     if (!facet->normal)
@@ -766,7 +886,7 @@ boolT qh_gram_schmidt(int dim, realT **row) {
 
   for (i=0; i < dim; i++) {
     rowi= row[i];
-    for (norm= 0.0, k= dim; k--; rowi++)
+    for (norm=0.0, k=dim; k--; rowi++)
       norm += *rowi * *rowi;
     norm= sqrt(norm);
     wmin_(Wmindenom, norm);
@@ -776,7 +896,7 @@ boolT qh_gram_schmidt(int dim, realT **row) {
       *(--rowi) /= norm;
     for (j=i+1; j < dim; j++) {
       rowj= row[j];
-      for (norm= 0.0, k=dim; k--; )
+      for (norm=0.0, k=dim; k--; )
         norm += *rowi++ * *rowj++;
       for (k=dim; k--; )
         *(--rowj) -= *(--rowi) * norm;
@@ -842,7 +962,7 @@ boolT qh_inthresholds(coordT *normal, realT *angle) {
 /*-<a                             href="qh-geom.htm#TOC"
   >-------------------------------</a><a name="joggleinput">-</a>
 
-  qh_joggleinput()
+  qh_joggleinput( )
     randomly joggle input to Qhull by qh.JOGGLEmax
     initial input is qh.first_point/qh.num_points of qh.hull_dim
       repeated calls use qh.input_points/qh.num_points
@@ -879,8 +999,8 @@ void qh_joggleinput(void) {
   if (!qh input_points) { /* first call */
     qh input_points= qh first_point;
     qh input_malloc= qh POINTSmalloc;
-    size= qh num_points * qh hull_dim * sizeof(coordT);
-    if (!(qh first_point=(coordT*)qh_malloc((size_t)size))) {
+    size= qh num_points * qh hull_dim * (int)sizeof(coordT);
+    if (!(qh first_point= (coordT *)qh_malloc((size_t)size))) {
       qh_fprintf(qh ferr, 6009, "qhull error: insufficient memory to joggle %d points\n",
           qh num_points);
       qh_errexit(qh_ERRmem, NULL, NULL);
@@ -903,14 +1023,14 @@ void qh_joggleinput(void) {
     qh_option("QJoggle", NULL, &qh JOGGLEmax);
   }
   if (qh build_cnt > 1 && qh JOGGLEmax > fmax_(qh MAXwidth/4, 0.1)) {
-      qh_fprintf(qh ferr, 6010, "qhull error: the current joggle for 'QJn', %.2g, is too large for the width\nof the input.  If possible, recompile Qhull with higher-precision reals.\n",
+      qh_fprintf(qh ferr, 6010, "qhull input error (qh_joggleinput): the current joggle for 'QJn', %.2g, is too large for the width\nof the input.  If possible, recompile Qhull with higher-precision reals.\n",
                 qh JOGGLEmax);
-      qh_errexit(qh_ERRqhull, NULL, NULL);
+      qh_errexit(qh_ERRinput, NULL, NULL);
   }
   /* for some reason, using qh ROTATErandom and qh_RANDOMseed does not repeat the run. Use 'TRn' instead */
   seed= qh_RANDOMint;
   qh_option("_joggle-seed", &seed, NULL);
-  trace0((qh ferr, 6, "qh_joggleinput: joggle input by %2.2g with seed %d\n",
+  trace0((qh ferr, 6, "qh_joggleinput: joggle input by %4.4g with seed %d\n",
     qh JOGGLEmax, seed));
   inputp= qh input_points;
   coordp= qh first_point;
@@ -994,12 +1114,12 @@ setT *qh_maxmin(pointT *points, int numpoints, int dimension) {
   && REALmax > 0.0 && -REALmax < 0.0)
     ; /* all ok */
   else {
-    qh_fprintf(qh ferr, 6011, "qhull error: floating point constants in user.h are wrong\n\
-REALepsilon %g REALmin %g REALmax %g -REALmax %g\n",
-             REALepsilon, REALmin, REALmax, -REALmax);
+    qh_fprintf(qh ferr, 6011, "qhull error: one or more floating point constants in user.h are inconsistent. REALmin %g, -REALmax %g, 0.0, REALepsilon %g, REALmax %g\n",
+          REALmin, -REALmax, REALepsilon, REALmax);
     qh_errexit(qh_ERRinput, NULL, NULL);
   }
   set= qh_settemp(2*dimension);
+  trace1((qh ferr, 8082, "qh_maxmin: dim             min             max           width    nearzero  min-point  max-point\n"));
   for (k=0; k < dimension; k++) {
     if (points == qh GOODpointp)
       minimum= maximum= points + dimension;
@@ -1018,7 +1138,7 @@ REALepsilon %g REALmin %g REALmax %g -REALmax %g\n",
       qh MAXlastcoord= maximum[k];
     }
     if (qh SCALElast && k == dimension-1)
-      maxcoord= qh MAXwidth;
+      maxcoord= qh MAXabs_coord;
     else {
       maxcoord= fmax_(maximum[k], -minimum[k]);
       if (qh GOODpointp) {
@@ -1030,23 +1150,28 @@ REALepsilon %g REALmin %g REALmax %g -REALmax %g\n",
     }
     maximize_(qh MAXabs_coord, maxcoord);
     qh MAXsumcoord += maxcoord;
-    qh_setappend(&set, maximum);
     qh_setappend(&set, minimum);
+    qh_setappend(&set, maximum);
     /* calculation of qh NEARzero is based on Golub & van Loan, 1983,
        Eq. 4.4-13 for "Gaussian elimination with complete pivoting".
        Golub & van Loan say that n^3 can be ignored and 10 be used in
        place of rho */
     qh NEARzero[k]= 80 * qh MAXsumcoord * REALepsilon;
+    trace1((qh ferr, 8106, "           %3d % 14.8e % 14.8e % 14.8e  %4.4e  p%-9d p%-d\n",
+            k, minimum[k], maximum[k], maximum[k]-minimum[k], qh NEARzero[k], qh_pointid(minimum), qh_pointid(maximum)));
+    if (qh SCALElast && k == dimension-1)
+      trace1((qh ferr, 8107, "           last coordinate scaled to (%4.4g, %4.4g), width %4.4e for option 'Qbb'\n",
+            qh MAXabs_coord - qh MAXwidth, qh MAXabs_coord, qh MAXwidth));
   }
-  if (qh IStracing >=1)
-    qh_printpoints(qh ferr, "qh_maxmin: found the max and min points(by dim):", set);
+  if (qh IStracing >= 1)
+    qh_printpoints(qh ferr, "qh_maxmin: found the max and min points (by dim):", set);
   return(set);
 } /* maxmin */
 
 /*-<a                             href="qh-geom.htm#TOC"
   >-------------------------------</a><a name="maxouter">-</a>
 
-  qh_maxouter()
+  qh_maxouter( )
     return maximum distance from facet to outer plane
     normally this is qh.max_outside+qh.DISTround
     does not include qh.JOGGLEmax
@@ -1056,6 +1181,7 @@ REALepsilon %g REALmin %g REALmax %g -REALmax %g\n",
 
   notes:
     need to add another qh.DISTround if testing actual point with computation
+    see qh_detmaxoutside for a qh_RATIO... target
 
   for joggle:
     qh_setfacetplane() updated qh.max_outer for Wnewvertexmax (max distance to vertex)
@@ -1068,7 +1194,7 @@ realT qh_maxouter(void) {
 
   dist= fmax_(qh max_outside, qh DISTround);
   dist += qh DISTround;
-  trace4((qh ferr, 4012, "qh_maxouter: max distance from facet to outer plane is %2.2g max_outside is %2.2g\n", dist, qh max_outside));
+  trace4((qh ferr, 4012, "qh_maxouter: max distance from facet to outer plane is %4.4g, qh.max_outside is %4.4g\n", dist, qh max_outside));
   return dist;
 } /* maxouter */
 
@@ -1077,32 +1203,45 @@ realT qh_maxouter(void) {
 
   qh_maxsimplex( dim, maxpoints, points, numpoints, simplex )
     determines maximum simplex for a set of points
-    starts from points already in simplex
+    maxpoints is the subset of points with a min or max coordinate
+    may start with points already in simplex
     skips qh.GOODpointp (assumes that it isn't in maxpoints)
 
   returns:
     simplex with dim+1 points
 
   notes:
-    assumes at least pointsneeded points in points
+    called by qh_initialvertices, qh_detvnorm, and qh_voronoi_center
+    requires qh.MAXwidth to estimate determinate for each vertex
+    assumes at least needed points in points
     maximizes determinate for x,y,z,w, etc.
     uses maxpoints as long as determinate is clearly non-zero
 
   design:
     initialize simplex with at least two points
       (find points with max or min x coordinate)
-    for each remaining dimension
-      add point that maximizes the determinate
-        (use points from maxpoints first)
+    create a simplex of dim+1 vertices as follows
+      add point from maxpoints that maximizes the determinate of the point and the simplex vertices  
+      if last point and maxdet/prevdet < qh_RATIOmaxsimplex (3.0e-2)
+        flag maybe_falsenarrow
+      if no maxpoint or maxnearzero or maybe_falsenarrow
+        search all points for maximum determinate
+        early exit if maybe_falsenarrow and !maxnearzero and maxdet > prevdet
 */
 void qh_maxsimplex(int dim, setT *maxpoints, pointT *points, int numpoints, setT **simplex) {
   pointT *point, **pointp, *pointtemp, *maxpoint, *minx=NULL, *maxx=NULL;
-  boolT nearzero, maxnearzero= False;
-  int k, sizinit;
-  realT maxdet= -REALmax, det, mincoord= REALmax, maxcoord= -REALmax;
+  boolT nearzero, maxnearzero= False, maybe_falsenarrow;
+  int i, sizinit;
+  realT maxdet= -1.0, prevdet= -1.0, det, mincoord= REALmax, maxcoord= -REALmax, mindet, ratio, targetdet;
 
+  if (qh MAXwidth <= 0.0) {
+    qh_fprintf(qh ferr, 6421, "qhull internal error (qh_maxsimplex): qh.MAXwidth required for qh_maxsimplex.  Used to estimate determinate\n");
+    qh_errexit(qh_ERRqhull, NULL, NULL);
+  }
   sizinit= qh_setsize(*simplex);
-  if (sizinit < 2) {
+  if (sizinit >= 2) {
+    maxdet= pow(qh MAXwidth, sizinit - 1);
+  }else {
     if (qh_setsize(maxpoints) >= 2) {
       FOREACHpoint_(maxpoints) {
         if (maxcoord < point[0]) {
@@ -1128,6 +1267,7 @@ void qh_maxsimplex(int dim, setT *maxpoints, pointT *points, int numpoints, setT
         }
       }
     }
+    maxdet= maxcoord - mincoord;
     qh_setunique(simplex, minx);
     if (qh_setsize(*simplex) < 2)
       qh_setunique(simplex, maxx);
@@ -1135,21 +1275,23 @@ void qh_maxsimplex(int dim, setT *maxpoints, pointT *points, int numpoints, setT
     if (sizinit < 2) {
       qh_joggle_restart("input has same x coordinate");
       if (zzval_(Zsetplane) > qh hull_dim+1) {
-        qh_fprintf(qh ferr, 6012, "qhull precision error (qh_maxsimplex for voronoi_center):\n%d points with the same x coordinate.\n",
-                 qh_setsize(maxpoints)+numpoints);
+        qh_fprintf(qh ferr, 6012, "qhull precision error (qh_maxsimplex for voronoi_center): %d points with the same x coordinate %4.4g\n",
+                 qh_setsize(maxpoints)+numpoints, mincoord);
         qh_errexit(qh_ERRprec, NULL, NULL);
       }else {
-        qh_fprintf(qh ferr, 6013, "qhull input error: input is less than %d-dimensional since it has the same x coordinate\n", qh hull_dim);
+        qh_fprintf(qh ferr, 6013, "qhull input error: input is less than %d-dimensional since all points have the same x coordinate %4.4g\n",
+                 qh hull_dim, mincoord);
         qh_errexit(qh_ERRinput, NULL, NULL);
       }
     }
   }
-  for (k=sizinit; k < dim+1; k++) {
+  for (i=sizinit; i < dim+1; i++) {
+    prevdet= maxdet;
     maxpoint= NULL;
-    maxdet= -REALmax;
+    maxdet= -1.0;
     FOREACHpoint_(maxpoints) {
-      if (!qh_setin(*simplex, point)) {
-        det= qh_detsimplex(point, *simplex, k, &nearzero);
+      if (!qh_setin(*simplex, point) && point != maxpoint) {
+        det= qh_detsimplex(point, *simplex, i, &nearzero); /* retests maxpoints if duplicate or multiple iterations */
         if ((det= fabs_(det)) > maxdet) {
           maxdet= det;
           maxpoint= point;
@@ -1157,23 +1299,41 @@ void qh_maxsimplex(int dim, setT *maxpoints, pointT *points, int numpoints, setT
         }
       }
     }
-    if (!maxpoint || maxnearzero) {
+    maybe_falsenarrow= False;
+    ratio= 1.0;
+    targetdet= prevdet * qh MAXwidth;
+    mindet= 10 * qh_RATIOmaxsimplex * targetdet;
+    if (maxdet > 0.0) {
+      ratio= maxdet / targetdet;
+      if (ratio < qh_RATIOmaxsimplex)
+        maybe_falsenarrow= True;
+    }
+    if (!maxpoint || maxnearzero || maybe_falsenarrow) {
       zinc_(Zsearchpoints);
       if (!maxpoint) {
-        trace0((qh ferr, 7, "qh_maxsimplex: searching all points for %d-th initial vertex.\n", k+1));
+        trace0((qh ferr, 7, "qh_maxsimplex: searching all points for %d-th initial vertex, better than mindet %4.4g, targetdet %4.4g\n",
+                i+1, mindet, targetdet));
+      }else if (qh ALLpoints) {
+        trace0((qh ferr, 30, "qh_maxsimplex: searching all points ('Qs') for %d-th initial vertex, better than p%d det %4.4g, targetdet %4.4g, ratio %4.4g\n",
+                i+1, qh_pointid(maxpoint), maxdet, targetdet, ratio));
+      }else if (maybe_falsenarrow) {
+        trace0((qh ferr, 17, "qh_maxsimplex: searching all points for %d-th initial vertex, better than p%d det %4.4g and mindet %4.4g, ratio %4.4g\n",
+                i+1, qh_pointid(maxpoint), maxdet, mindet, ratio));
       }else {
-        trace0((qh ferr, 8, "qh_maxsimplex: searching all points for %d-th initial vertex, better than p%d det %2.2g\n",
-                k+1, qh_pointid(maxpoint), maxdet));
+        trace0((qh ferr, 8, "qh_maxsimplex: searching all points for %d-th initial vertex, better than p%d det %2.2g and mindet %4.4g, targetdet %4.4g\n",
+                i+1, qh_pointid(maxpoint), maxdet, mindet, targetdet));
       }
       FORALLpoint_(points, numpoints) {
         if (point == qh GOODpointp)
           continue;
-        if (!qh_setin(*simplex, point)) {
-          det= qh_detsimplex(point, *simplex, k, &nearzero);
+        if (!qh_setin(maxpoints, point) && !qh_setin(*simplex, point)) {
+          det= qh_detsimplex(point, *simplex, i, &nearzero);
           if ((det= fabs_(det)) > maxdet) {
             maxdet= det;
             maxpoint= point;
             maxnearzero= nearzero;
+            if (!maxnearzero && !qh ALLpoints && maxdet > mindet)
+              break;
           }
         }
       }
@@ -1183,9 +1343,9 @@ void qh_maxsimplex(int dim, setT *maxpoints, pointT *points, int numpoints, setT
       qh_errexit(qh_ERRqhull, NULL, NULL);
     }
     qh_setappend(simplex, maxpoint);
-    trace1((qh ferr, 1002, "qh_maxsimplex: selected point p%d for %d`th initial vertex, det=%2.2g\n",
-            qh_pointid(maxpoint), k+1, maxdet));
-  } /* k */
+    trace1((qh ferr, 1002, "qh_maxsimplex: selected point p%d for %d`th initial vertex, det=%4.4g, targetdet=%4.4g, mindet=%4.4g\n",
+            qh_pointid(maxpoint), i+1, maxdet, prevdet * qh MAXwidth, mindet));
+  } /* i */
 } /* maxsimplex */
 
 /*-<a                             href="qh-geom.htm#TOC"
@@ -1385,7 +1545,7 @@ void qh_printpoints(FILE *fp, const char *string, setT *points) {
 /*-<a                             href="qh-geom.htm#TOC"
   >-------------------------------</a><a name="projectinput">-</a>
 
-  qh_projectinput()
+  qh_projectinput( )
     project input points using qh.lower_bound/upper_bound and qh DELAUNAY
     if qh.lower_bound[k]=qh.upper_bound[k]= 0,
       removes dimension k
@@ -1425,14 +1585,14 @@ void qh_projectinput(void) {
   int k,i;
   int newdim= qh input_dim, newnum= qh num_points;
   signed char *project;
-  int projectsize= (qh input_dim+1)*sizeof(*project);
+  int projectsize= (qh input_dim + 1) * (int)sizeof(*project);
   pointT *newpoints, *coord, *infinity;
   realT paraboloid, maxboloid= 0;
 
-  project= (signed char*)qh_memalloc(projectsize);
-  memset((char*)project, 0, (size_t)projectsize);
+  project= (signed char *)qh_memalloc(projectsize);
+  memset((char *)project, 0, (size_t)projectsize);
   for (k=0; k < qh input_dim; k++) {   /* skip Delaunay bound */
-    if (qh lower_bound[k] == 0 && qh upper_bound[k] == 0) {
+    if (qh lower_bound[k] == 0.0 && qh upper_bound[k] == 0.0) {
       project[k]= -1;
       newdim--;
     }
@@ -1448,7 +1608,7 @@ void qh_projectinput(void) {
     qh_fprintf(qh ferr, 6015, "qhull internal error (qh_projectinput): dimension after projection %d != hull_dim %d\n", newdim, qh hull_dim);
     qh_errexit(qh_ERRqhull, NULL, NULL);
   }
-  if (!(newpoints= qh temp_malloc= (coordT*)qh_malloc(newnum*newdim*sizeof(coordT)))){
+  if (!(newpoints= qh temp_malloc= (coordT *)qh_malloc((size_t)(newnum * newdim) * sizeof(coordT)))) {
     qh_memfree(project, projectsize);
     qh_fprintf(qh ferr, 6016, "qhull error: insufficient memory to project %d points\n",
            qh num_points);
@@ -1499,7 +1659,7 @@ void qh_projectinput(void) {
     qh num_points++;
     trace0((qh ferr, 9, "qh_projectinput: projected points to paraboloid for Delaunay\n"));
   }else if (qh DELAUNAY)  /* !qh ATinfinity */
-    qh_setdelaunay( qh hull_dim, qh num_points, qh first_point);
+    qh_setdelaunay(qh hull_dim, qh num_points, qh first_point);
 } /* projectinput */
 
 
@@ -1613,12 +1773,12 @@ void qh_rotatepoints(realT *points, int numpoints, int dim, realT **row) {
 
   if (qh IStracing >= 1)
     qh_printmatrix(qh ferr, "qh_rotatepoints: rotate points by", row, dim, dim);
-  for (point= points, j= numpoints; j--; point += dim) {
+  for (point=points, j=numpoints; j--; point += dim) {
     newval= row[dim];
     for (i=0; i < dim; i++) {
       rowi= row[i];
       coord= point;
-      for (sum= 0.0, k= dim; k--; )
+      for (sum=0.0, k=dim; k--; )
         sum += *rowi++ * *coord++;
       *(newval++)= sum;
     }
@@ -1631,7 +1791,7 @@ void qh_rotatepoints(realT *points, int numpoints, int dim, realT **row) {
 /*-<a                             href="qh-geom.htm#TOC"
   >-------------------------------</a><a name="scaleinput">-</a>
 
-  qh_scaleinput()
+  qh_scaleinput( )
     scale input points using qh low_bound/high_bound
     input points given by qh first_point, num_points, hull_dim
     if qh POINTSmalloc, overwrites input points, else mallocs a new array
@@ -1657,16 +1817,20 @@ void qh_scaleinput(void) {
   >-------------------------------</a><a name="scalelast">-</a>
 
   qh_scalelast( points, numpoints, dim, low, high, newhigh )
-    scale last coordinate to [0,m] for Delaunay triangulations
+    scale last coordinate to [0.0, newhigh], for Delaunay triangulation
     input points given by points, numpoints, dim
 
   returns:
-    changes scale of last coordinate from [low, high] to [0, newhigh]
+    changes scale of last coordinate from [low, high] to [0.0, newhigh]
     overwrites last coordinate of each point
     saves low/high/newhigh in qh.last_low, etc. for qh_setdelaunay()
 
   notes:
-    when called by qh_setdelaunay, low/high may not match actual data
+    to reduce precision issues, qh_scalelast makes the last coordinate similar to other coordinates
+      the last coordinate for Delaunay triangulation is the sum of squares of input coordinates
+      note that the range [0.0, newwidth] is wrong for narrow distributions with large positive coordinates (e.g., [995933.64, 995963.48])
+
+    when called by qh_setdelaunay, low/high may not match the data passed to qh_setdelaunay
 
   design:
     compute scale and shift factors
@@ -1675,26 +1839,28 @@ void qh_scaleinput(void) {
 void qh_scalelast(coordT *points, int numpoints, int dim, coordT low,
                    coordT high, coordT newhigh) {
   realT scale, shift;
-  coordT *coord;
+  coordT *coord, newlow;
   int i;
   boolT nearzero= False;
 
-  trace4((qh ferr, 4013, "qh_scalelast: scale last coordinate from [%2.2g, %2.2g] to [0,%2.2g]\n",
-    low, high, newhigh));
+  newlow= 0.0;
+  trace4((qh ferr, 4013, "qh_scalelast: scale last coordinate from [%2.2g, %2.2g] to [%2.2g, %2.2g]\n",
+    low, high, newlow, newhigh));
   qh last_low= low;
   qh last_high= high;
   qh last_newhigh= newhigh;
-  scale= qh_divzero(newhigh, high - low,
+  scale= qh_divzero(newhigh - newlow, high - low,
                   qh MINdenom_1, &nearzero);
   if (nearzero) {
     if (qh DELAUNAY)
-      qh_fprintf(qh ferr, 6019, "qhull input error: can not scale last coordinate.  Input is cocircular\n   or cospherical.   Use option 'Qz' to add a point at infinity.\n");
+      qh_fprintf(qh ferr, 6019, "qhull input error (qh_scalelast): can not scale last coordinate to [%4.4g, %4.4g].  Input is cocircular or cospherical.   Use option 'Qz' to add a point at infinity.\n",
+             newlow, newhigh);
     else
-      qh_fprintf(qh ferr, 6020, "qhull input error: can not scale last coordinate.  New bounds [0, %2.2g] are too wide for\nexisting bounds [%2.2g, %2.2g] (width %2.2g)\n",
-                newhigh, low, high, high-low);
+      qh_fprintf(qh ferr, 6020, "qhull input error (qh_scalelast): can not scale last coordinate to [%4.4g, %4.4g].  New bounds are too wide for compared to existing bounds [%4.4g, %4.4g] (width %4.4g)\n",
+             newlow, newhigh, low, high, high-low);
     qh_errexit(qh_ERRinput, NULL, NULL);
   }
-  shift= - low * newhigh / (high-low);
+  shift= newlow - low * scale;
   coord= points + dim - 1;
   for (i=numpoints; i--; coord += dim)
     *coord= *coord * scale + shift;
@@ -1817,7 +1983,7 @@ void qh_setdelaunay(int dim, int count, pointT *points) {
       coord= *coordp++;
       paraboloid += coord*coord;
     }
-    *coordp++ = paraboloid;
+    *coordp++= paraboloid;
   }
   if (qh last_low < REALmax/2)
     qh_scalelast(points, count, dim, qh last_low, qh last_high, qh last_newhigh);
@@ -1866,6 +2032,7 @@ boolT qh_sethalfspace(int dim, coordT *coords, coordT **nextp,
     }
   }
   *nextp= coordp;
+#ifndef qh_NOtrace
   if (qh IStracing >= 4) {
     qh_fprintf(qh ferr, 8021, "qh_sethalfspace: halfspace at offset %6.2g to point: ", *offset);
     for (k=dim, coordp=coords; k--; ) {
@@ -1874,6 +2041,7 @@ boolT qh_sethalfspace(int dim, coordT *coords, coordT **nextp,
     }
     qh_fprintf(qh ferr, 8023, "\n");
   }
+#endif
   return True;
 LABELerroroutside:
   feasiblep= feasible;
@@ -1909,8 +2077,8 @@ LABELerroroutside:
     call before qh_init_B or qh_initqhull_globals
     free memory when done
     unused/untested code: please email bradb@shore.net if this works ok for you
-    if using option 'Fp', qh->feasible_point must be set (e.g., to 'feasible')
-    qh->feasible_point is a malloc'd array that is freed by qh_freebuffers.
+    if using option 'Fp', qh.feasible_point must be set (e.g., to 'feasible')
+    qh feasible_point is a malloc'd array that is freed by qh_freebuffers.
 
   design:
     see qh_sethalfspace
@@ -1922,7 +2090,7 @@ coordT *qh_sethalfspace_all(int dim, int count, coordT *halfspaces, pointT *feas
 
   trace0((qh ferr, 12, "qh_sethalfspace_all: compute dual for halfspace intersection\n"));
   newdim= dim - 1;
-  if (!(newpoints=(coordT*)qh_malloc(count*newdim*sizeof(coordT)))){
+  if (!(newpoints= (coordT *)qh_malloc((size_t)(count * newdim) * sizeof(coordT)))){
     qh_fprintf(qh ferr, 6024, "qhull error: insufficient memory to compute dual of %d halfspaces\n",
           count);
     qh_errexit(qh_ERRmem, NULL, NULL);
@@ -1945,7 +2113,7 @@ coordT *qh_sethalfspace_all(int dim, int count, coordT *halfspaces, pointT *feas
 /*-<a                             href="qh-geom.htm#TOC"
   >-------------------------------</a><a name="sharpnewfacets">-</a>
 
-  qh_sharpnewfacets()
+  qh_sharpnewfacets( )
 
   returns:
     true if could be an acute angle (facets in different quadrants)
@@ -1960,10 +2128,10 @@ coordT *qh_sethalfspace_all(int dim, int count, coordT *halfspaces, pointT *feas
 */
 boolT qh_sharpnewfacets(void) {
   facetT *facet;
-  boolT issharp = False;
+  boolT issharp= False;
   int *quadrant, k;
 
-  quadrant= (int*)qh_memalloc(qh hull_dim * sizeof(int));
+  quadrant= (int *)qh_memalloc(qh hull_dim * (int)sizeof(int));
   FORALLfacet_(qh newfacet_list) {
     if (facet == qh newfacet_list) {
       for (k=qh hull_dim; k--; )
@@ -1979,11 +2147,49 @@ boolT qh_sharpnewfacets(void) {
     if (issharp)
       break;
   }
-  qh_memfree( quadrant, qh hull_dim * sizeof(int));
+  qh_memfree(quadrant, qh hull_dim * (int)sizeof(int));
   trace3((qh ferr, 3001, "qh_sharpnewfacets: %d\n", issharp));
   return issharp;
 } /* sharpnewfacets */
 
+/*-<a                             href="qh-geom.htm#TOC"
+  >-------------------------------</a><a name="vertex_bestdist">-</a>
+
+  qh_vertex_bestdist( vertices )
+  qh_vertex_bestdist2( vertices, vertexp, vertexp2 )
+    return nearest distance between vertices
+    optionally returns vertex and vertex2
+
+  notes:
+    called by qh_partitioncoplanar, qh_mergefacet, qh_check_maxout, qh_checkpoint
+*/
+coordT qh_vertex_bestdist(setT *vertices) {
+  vertexT *vertex, *vertex2;
+
+  return qh_vertex_bestdist2(vertices, &vertex, &vertex2);
+} /* vertex_bestdist */
+
+coordT qh_vertex_bestdist2(setT *vertices, vertexT **vertexp/*= NULL*/, vertexT **vertexp2/*= NULL*/) {
+  vertexT *vertex, *vertexA, *bestvertex= NULL, *bestvertex2= NULL;
+  coordT dist, bestdist= REALmax;
+  int k, vertex_i, vertex_n;
+
+  FOREACHvertex_i_(vertices) {
+    for (k= vertex_i+1; k < vertex_n; k++) {
+      vertexA= SETelemt_(vertices, k, vertexT);
+      dist= qh_pointdist(vertex->point, vertexA->point, -qh hull_dim);
+      if (dist < bestdist) {
+        bestdist= dist;
+        bestvertex= vertex;
+        bestvertex2= vertexA;
+      }
+    }
+  }
+  *vertexp= bestvertex;
+  *vertexp2= bestvertex2;
+  return sqrt(bestdist);
+} /* vertex_bestdist */
+
 /*-<a                             href="qh-geom.htm#TOC"
   >-------------------------------</a><a name="voronoi_center">-</a>
 
@@ -2011,7 +2217,7 @@ boolT qh_sharpnewfacets(void) {
 */
 pointT *qh_voronoi_center(int dim, setT *points) {
   pointT *point, **pointp, *point0;
-  pointT *center= (pointT*)qh_memalloc(qh center_size);
+  pointT *center= (pointT *)qh_memalloc(qh center_size);
   setT *simplex;
   int i, j, k, size= qh_setsize(points);
   coordT *gmcoord;
@@ -2022,7 +2228,7 @@ pointT *qh_voronoi_center(int dim, setT *points) {
     simplex= points;
   else if (size < dim+1) {
     qh_memfree(center, qh center_size);
-    qh_fprintf(qh ferr, 6025, "qhull internal error (qh_voronoi_center):\n  need at least %d points to construct a Voronoi center\n",
+    qh_fprintf(qh ferr, 6025, "qhull internal error (qh_voronoi_center):  need at least %d points to construct a Voronoi center\n",
              dim+1);
     qh_errexit(qh_ERRqhull, NULL, NULL);
     simplex= points;  /* never executed -- avoids warning */
@@ -2075,7 +2281,7 @@ pointT *qh_voronoi_center(int dim, setT *points) {
     }
 #ifndef qh_NOtrace
     if (qh IStracing >= 3) {
-      qh_fprintf(qh ferr, 8033, "qh_voronoi_center: det %2.2g factor %2.2g ", det, factor);
+      qh_fprintf(qh ferr, 3061, "qh_voronoi_center: det %2.2g factor %2.2g ", det, factor);
       qh_printmatrix(qh ferr, "center:", &center, 1, dim);
       if (qh IStracing >= 5) {
         qh_printpoints(qh ferr, "points", simplex);
