@@ -11,12 +11,14 @@
    either the actual size of the set plus 1, or the NULL terminator
    of the set (i.e., setelemT).
 
-   Copyright (c) 1993-2018 The Geometry Center.
-   $Id: //main/2015/qhull/src/libqhull/qset.c#5 $$Change: 2549 $
-   $DateTime: 2018/12/28 22:24:20 $$Author: bbarber $
+   Do not reference 'qh' since it brings in qhT unnecessarily
+
+   Copyright (c) 1993-2019 The Geometry Center.
+   $Id: //main/2019/qhull/src/libqhull/qset.c#6 $$Change: 2711 $
+   $DateTime: 2019/06/27 22:34:56 $$Author: bbarber $
 */
 
-#include "user.h" /* for QHULL_CRTDBG */
+#include "libqhull.h" /* for qhT and QHULL_CRTDBG */
 #include "qset.h"
 #include "mem.h"
 #include <stdio.h>
@@ -44,7 +46,7 @@ void    qh_fprintf(FILE *fp, int msgcode, const char *fmt, ... );
 /*-<a                             href="qh-set.htm#TOC"
   >--------------------------------<a name="setaddnth">-</a>
 
-  qh_setaddnth( setp, nth, newelem)
+  qh_setaddnth( setp, nth, newelem )
     adds newelem as n'th element of sorted or unsorted *setp
 
   notes:
@@ -116,7 +118,7 @@ void qh_setaddsorted(setT **setp, void *newelem) {
 /*-<a                             href="qh-set.htm#TOC"
   >-------------------------------<a name="setappend">-</a>
 
-  qh_setappend( setp, newelem)
+  qh_setappend( setp, newelem )
     append newelem to *setp
 
   notes:
@@ -148,7 +150,7 @@ void qh_setappend(setT **setp, void *newelem) {
 /*-<a                             href="qh-set.htm#TOC"
   >-------------------------------<a name="setappend_set">-</a>
 
-  qh_setappend_set( setp, setA)
+  qh_setappend_set( setp, setA )
     appends setA to *setp
 
   notes:
@@ -231,7 +233,7 @@ void qh_setappend2ndlast(setT **setp, void *newelem) {
   design:
     checks that maxsize, actual size, and NULL terminator agree
 */
-void qh_setcheck(setT *set, const char *tname, unsigned id) {
+void qh_setcheck(setT *set, const char *tname, unsigned int id) {
   int maxsize, size;
   int waserr= 0;
 
@@ -283,7 +285,7 @@ void qh_setcompact(setT *set) {
   destp= elemp= firstp= SETaddr_(set, void);
   endp= destp + size;
   while (1) {
-    if (!(*destp++ = *elemp++)) {
+    if (!(*destp++= *elemp++)) {
       destp--;
       if (elemp > endp)
         break;
@@ -324,7 +326,7 @@ setT *qh_setcopy(setT *set, int extra) {
 /*-<a                             href="qh-set.htm#TOC"
   >-------------------------------<a name="setdel">-</a>
 
-  qh_setdel( set, oldelem )
+  qh_setdel(set, oldelem )
     delete oldelem from an unsorted set
 
   returns:
@@ -367,7 +369,7 @@ void *qh_setdel(setT *set, void *oldelem) {
 /*-<a                             href="qh-set.htm#TOC"
   >-------------------------------<a name="setdellast">-</a>
 
-  qh_setdellast( set)
+  qh_setdellast( set )
     return last element of set or NULL
 
   notes:
@@ -428,7 +430,7 @@ void *qh_setdelnth(setT *set, int nth) {
 
   sizep= SETsizeaddr_(set);
   if ((sizep->i--)==0)         /*  if was a full set */
-      sizep->i= set->maxsize;  /*     *sizep= (maxsize-1)+ 1 */
+    sizep->i= set->maxsize;    /*    *sizep= (maxsize-1)+ 1 */
   if (nth < 0 || nth >= sizep->i) {
     qh_fprintf(qhmem.ferr, 6174, "qhull internal error (qh_setdelnth): nth %d is out-of-bounds for set:\n", nth);
     qh_setprint(qhmem.ferr, "", set);
@@ -600,7 +602,7 @@ int qh_setequal(setT *setA, setT *setB) {
     return 1;
   elemAp= SETaddr_(setA, void);
   elemBp= SETaddr_(setB, void);
-  if (!memcmp((char *)elemAp, (char *)elemBp, sizeA*SETelemsize))
+  if (!memcmp((char *)elemAp, (char *)elemBp, (size_t)(sizeA * SETelemsize)))
     return 1;
   return 0;
 } /* setequal */
@@ -718,7 +720,7 @@ void qh_setfree(setT **setp) {
   void **freelistp;  /* used if !qh_NOmem by qh_memfree_() */
 
   if (*setp) {
-    size= sizeof(setT) + ((*setp)->maxsize)*SETelemsize;
+    size= (int)sizeof(setT) + ((*setp)->maxsize)*SETelemsize;
     if (size <= qhmem.LASTsize) {
       qh_memfree_(*setp, size, freelistp);
     }else
@@ -771,7 +773,7 @@ void qh_setfreelong(setT **setp) {
   int size;
 
   if (*setp) {
-    size= sizeof(setT) + ((*setp)->maxsize)*SETelemsize;
+    size= (int)sizeof(setT) + ((*setp)->maxsize)*SETelemsize;
     if (size > qhmem.LASTsize) {
       qh_memfree(*setp, size);
       *setp= NULL;
@@ -806,7 +808,7 @@ int qh_setin(setT *set, void *setelem) {
 /*-<a                             href="qh-set.htm#TOC"
   >-------------------------------<a name="setindex">-</a>
 
-  qh_setindex( set, atelem )
+  qh_setindex(set, atelem )
     returns the index of atelem in set.
     returns -1, if not in set or maxsize wrong
 
@@ -843,7 +845,13 @@ int qh_setindex(setT *set, void *atelem) {
     returns a larger set that contains all elements of *oldsetp
 
   notes:
-    the set is at least twice as large
+    if long memory,
+      the new set is 2x larger
+    if qhmem.LASTsize is between 1.5x and 2x
+      the new set is qhmem.LASTsize
+    otherwise use quick memory,
+      the new set is 2x larger, rounded up to next qh_memsize
+       
     if temp set, updates qhmem.tempstack
 
   design:
@@ -853,22 +861,23 @@ int qh_setindex(setT *set, void *atelem) {
     deletes the old set
 */
 void qh_setlarger(setT **oldsetp) {
-  int size= 1;
+  int setsize= 1, newsize;
   setT *newset, *set, **setp, *oldset;
   setelemT *sizep;
   setelemT *newp, *oldp;
 
   if (*oldsetp) {
     oldset= *oldsetp;
-    SETreturnsize_(oldset, size);
+    SETreturnsize_(oldset, setsize);
     qhmem.cntlarger++;
-    qhmem.totlarger += size+1;
-    newset= qh_setnew(2 * size);
+    qhmem.totlarger += setsize+1;
+    qh_setlarger_quick(setsize, &newsize);
+    newset= qh_setnew(newsize);
     oldp= (setelemT *)SETaddr_(oldset, void);
     newp= (setelemT *)SETaddr_(newset, void);
-    memcpy((char *)newp, (char *)oldp, (size_t)(size+1) * SETelemsize);
+    memcpy((char *)newp, (char *)oldp, (size_t)(setsize+1) * SETelemsize);
     sizep= SETsizeaddr_(newset);
-    sizep->i= size+1;
+    sizep->i= setsize+1;
     FOREACHset_((setT *)qhmem.tempstack) {
       if (set == oldset)
         *(setp-1)= newset;
@@ -880,6 +889,39 @@ void qh_setlarger(setT **oldsetp) {
 } /* setlarger */
 
 
+/*-<a                             href="qh-set.htm#TOC"
+  >-------------------------------<a name="setlarger_quick">-</a>
+
+  qh_setlarger_quick( setsize, newsize )
+    determine newsize for setsize
+    returns True if newsize fits in quick memory
+
+  design:
+    if 2x fits into quick memory
+      return True, 2x
+    if x+4 does not fit into quick memory
+      return False, 2x
+    if x+x/3 fits into quick memory
+      return True, the last quick set
+    otherwise
+      return False, 2x
+*/
+int qh_setlarger_quick(int setsize, int *newsize) {
+    int lastquickset;
+
+    *newsize= 2 * setsize;
+    lastquickset= (qhmem.LASTsize - (int)sizeof(setT)) / SETelemsize; /* matches size computation in qh_setnew */
+    if (*newsize <= lastquickset)
+      return 1;
+    if (setsize + 4 > lastquickset)
+      return 0;
+    if (setsize + setsize/3 <= lastquickset) {
+      *newsize= lastquickset;
+      return 1;
+    }
+    return 0;
+} /* setlarger_quick */
+
 /*-<a                             href="qh-set.htm#TOC"
   >-------------------------------<a name="setlast">-</a>
 
@@ -929,7 +971,7 @@ setT *qh_setnew(int setsize) {
 
   if (!setsize)
     setsize++;
-  size= sizeof(setT) + setsize * SETelemsize;
+  size= (int)sizeof(setT) + setsize * SETelemsize; /* setT includes NULL terminator, see qh.LASTquickset */
   if (size>0 && size <= qhmem.LASTsize) {
     qh_memalloc_(size, freelistp, set, setT);
 #ifndef qh_NOmem
@@ -938,7 +980,7 @@ setT *qh_setnew(int setsize) {
       setsize += (sizereceived - size)/SETelemsize;
 #endif
   }else
-    set= (setT*)qh_memalloc(size);
+    set= (setT *)qh_memalloc(size);
   set->maxsize= setsize;
   set->e[setsize].i= 1;
   set->e[0].p= NULL;
@@ -1104,6 +1146,7 @@ void qh_setreplace(setT *set, void *oldelem, void *newelem) {
     errors if set's maxsize is incorrect
     same as SETreturnsize_(set)
     same code for qh_setsize [qset.c] and QhullSetBase::count
+    if first element is NULL, SETempty_() is True but qh_setsize may be greater than 0
 
   design:
     determine actual size of set from maxsize
@@ -1132,7 +1175,7 @@ int qh_setsize(setT *set) {
   >-------------------------------<a name="settemp">-</a>
 
   qh_settemp( setsize )
-    return a stacked, temporary set of upto setsize elements
+    return a stacked, temporary set of up to setsize elements
 
   notes:
     use settempfree or settempfree_all to release from qhmem.tempstack
@@ -1190,7 +1233,7 @@ void qh_settempfree(setT **set) {
 /*-<a                             href="qh-set.htm#TOC"
   >-------------------------------<a name="settempfree_all">-</a>
 
-  qh_settempfree_all(  )
+  qh_settempfree_all( )
     free all temporary sets in qhmem.tempstack
 
   design:
@@ -1209,7 +1252,7 @@ void qh_settempfree_all(void) {
 /*-<a                             href="qh-set.htm#TOC"
   >-------------------------------<a name="settemppop">-</a>
 
-  qh_settemppop(  )
+  qh_settemppop( )
     pop and return temporary set from qhmem.tempstack
 
   notes:
@@ -1221,7 +1264,7 @@ void qh_settempfree_all(void) {
 setT *qh_settemppop(void) {
   setT *stackedset;
 
-  stackedset= (setT*)qh_setdellast(qhmem.tempstack);
+  stackedset= (setT *)qh_setdellast(qhmem.tempstack);
   if (!stackedset) {
     qh_fprintf(qhmem.ferr, 6180, "qhull internal error (qh_settemppop): pop from empty temporary stack\n");
     qh_errexit(qhmem_ERRqhull, NULL, NULL);
