@@ -1,12 +1,12 @@
-/*<html><pre>  -<a                             href="../libqhull/index.htm#TOC"
+/*<html><pre>  -<a                             href="../libqhull_r/index.htm#TOC"
   >-------------------------------</a><a name="TOP">-</a>
 
-   testset.c -- test qset.c and its use of mem.c
+   testset_r.c -- test qset_r.c and its use of mem_r.c
 
    The test sets are pointers to int.  Normally a set is a pointer to a type (e.g., facetT, ridgeT, etc.).
    For consistency in notation, an "int" is typedef'd to i2T
 
-Functions and macros from qset.h.  Counts occurrences in this test.  Does not correspond to thoroughness.
+Functions and macros from qset_r.h.  Counts occurrences in this test.  Does not correspond to thoroughness.
     qh_setaddsorted -- 4 tests
     qh_setaddnth -- 1 test
     qh_setappend -- 7 tests
@@ -66,9 +66,9 @@ Functions and macros from qset.h.  Counts occurrences in this test.  Does not co
     SETsecondt_ -- 2 tests
     SETtruncate_ -- 2 tests
 
-    Copyright (c) 2012-2018 C.B. Barber. All rights reserved.
-    $Id: //main/2015/qhull/src/testqset_r/testqset_r.c#7 $$Change: 2549 $
-    $DateTime: 2018/12/28 22:24:20 $$Author: bbarber $
+    Copyright (c) 2012-2019 C.B. Barber. All rights reserved.
+    $Id: //main/2019/qhull/src/testqset_r/testqset_r.c#4 $$Change: 2683 $
+    $DateTime: 2019/06/14 16:05:16 $$Author: bbarber $
 */
 
 #include "libqhull_r/user_r.h"  /* QHULL_CRTDBG */
@@ -84,14 +84,14 @@ Functions and macros from qset.h.  Counts occurrences in this test.  Does not co
 typedef int i2T;
 #define MAXerrorCount 100 /* quit after n errors */
 
-#define FOREACHint_( ints ) FOREACHsetelement_( i2T, ints, i2)
-#define FOREACHint4_( ints ) FOREACHsetelement_( i2T, ints, i4)
-#define FOREACHint_i_( qh, ints ) FOREACHsetelement_i_( qh, i2T, ints, i2)
-#define FOREACHintreverse_( qh, ints ) FOREACHsetelementreverse_( qh, i2T, ints, i2)
-#define FOREACHintreverse12_( ints ) FOREACHsetelementreverse12_( i2T, ints, i2)
+#define FOREACHint_( ints ) FOREACHsetelement_(i2T, ints, i2)
+#define FOREACHint_i_(qh, ints) FOREACHsetelement_i_(qh, i2T, ints, i2)
+/* not tested -- #define FOREACHint4_( ints ) FOREACHsetelement_(i2T, ints, i4) */
+/* not tested -- #define FOREACHintreverse_(qh, ints) FOREACHsetelementreverse_(qh, i2T, ints, i2) */
+/* not tested -- #define FOREACHintreverse12_( ints ) FOREACHsetelementreverse12_(i2T, ints, i2) */
 
 enum {
-    MAXint= 0x7fffffff,
+    MAXint= 0x7fffffff
 };
 
 char prompt[]= "testqset_r N [M] [T5] -- Test reentrant qset_r.c and mem_r.c\n\
@@ -109,7 +109,7 @@ char prompt[]= "testqset_r N [M] [T5] -- Test reentrant qset_r.c and mem_r.c\n\
 
 int error_count= 0;  /* Global error_count.  checkSetContents(qh) keeps its own error count.  It exits on too many errors */
 
-/* Macros normally defined in geom.h */
+/* Macros normally defined in geom_r.h */
 #define fmax_( a,b )  ( ( a ) < ( b ) ? ( b ) : ( a ) )
 
 /* Macros normally defined in QhullSet.h */
@@ -121,8 +121,9 @@ void    qh_fprintf_stderr(int msgcode, const char *fmt, ... );
 void    qh_free(void *mem);
 void   *qh_malloc(size_t size);
 
-/* Normally defined in user_r.c */
+/* Normally defined in user_r.c, use void instead of facetT/ridgeT for testqset_r.c */
 
+void    qh_errexit(qhT *qh, int exitcode, facetT *f, ridgeT *r);
 void    qh_errexit(qhT *qh, int exitcode, facetT *f, ridgeT *r)
 {
     (void)f; /* unused */
@@ -131,8 +132,9 @@ void    qh_errexit(qhT *qh, int exitcode, facetT *f, ridgeT *r)
     qh_exit(exitcode);
 }
 
-/* Normally defined in userprintf.c */
+/* Normally defined in userprintf_r.c */
 
+void    qh_fprintf(qhT *qh, FILE *fp, int msgcode, const char *fmt, ... );
 void    qh_fprintf(qhT *qh, FILE *fp, int msgcode, const char *fmt, ... )
 {
     static int needs_cr= 0;  /* True if qh_fprintf needs a CR. testqset_r is not itself reentrant */
@@ -143,10 +145,10 @@ void    qh_fprintf(qhT *qh, FILE *fp, int msgcode, const char *fmt, ... )
     if (!fp) {
         /* Do not use qh_fprintf_stderr.  This is a standalone program */
         if(!qh)
-            fprintf(stderr, "QH6241 qh_fprintf: fp and qh not defined for '%s'", fmt);
+            fprintf(stderr, "QH6043 testqset_r (qh_fprintf): fp and qh not defined for '%s'\n", fmt);
         else
-            fprintf(stderr, "QH6232 qh_fprintf: fp is 0.  Was wrong qh_fprintf called for '%s'", fmt);
-        qh_errexit(qh, 6232, NULL, NULL);
+            fprintf(stderr, "QH6232 testqset_r (qh_fprintf): fp is 0.  Was wrong qh_fprintf called for '%s'\n", fmt);
+        qh_errexit(qh, qh_ERRqhull, NULL, NULL);
     }
     if(fmtlen>0){
         if(fmt[fmtlen-1]=='\n'){
@@ -159,7 +161,7 @@ void    qh_fprintf(qhT *qh, FILE *fp, int msgcode, const char *fmt, ... )
         }
     }
     if(msgcode>=6000 && msgcode<7000){
-        fprintf(fp, "Error TQ%d ", msgcode);
+        fprintf(fp, "Error QH%d ", msgcode);
     }
     va_start(args, fmt);
     vfprintf(fp, fmt, args);
@@ -187,7 +189,7 @@ int main(int argc, char **argv) {
     int *intarray= NULL;
     int numInts;
     int checkEvery= MAXint;
-    int curlong, totlong;
+    int curlong, totlong;  /* used if !qh_NOmem */
     int traceLevel= 4; /* 4 normally, no tracing since qset does not log.  Option 'T5' for memory tracing */
     qhT qh_qh;
     qhT *qh= &qh_qh;
@@ -209,17 +211,21 @@ int main(int argc, char **argv) {
     testSettemp(qh, numInts, intarray, checkEvery);
     testSetlastEtc(qh, numInts, intarray, checkEvery);
     testSetdelsortedEtc(qh, numInts, intarray, checkEvery);
-    printf("\n\nNot testing qh_setduplicate and qh_setfree2.\n  These routines use heap-allocated set contents.  See qhull tests.\n");
+    qh_fprintf(qh, stderr, 8083, "\nNot testing qh_setduplicate and qh_setfree2.  These routines use heap-allocated,\n\
+set contents.  See qhull tests in eg/q_test and bin/qhulltest.\n\n");
 
-    qh_memstatistics(qh, stdout);
+    qh_memstatistics(qh, stderr);
+#ifndef qh_NOmem
     qh_memfreeshort(qh, &curlong, &totlong);
     if (curlong || totlong){
-        qh_fprintf(qh, stderr, 8043, "qh_memfreeshort: did not free %d bytes of long memory(%d pieces)\n", totlong, curlong);
+        qh_fprintf(qh, stderr, 8084, "testqset_r: did not free %d bytes of long memory(%d pieces)\n", totlong, curlong);
         error_count++;
     }
+#endif
+    fflush(NULL);
     if(error_count){
-        qh_fprintf(qh, stderr, 8012, "testqset: %d errors\n\n", error_count);
-        exit(1);
+        qh_fprintf(qh, stderr, 8088, "testqset_r: %d errors\n\n", error_count);
+        exit(qh_ERRqhull);
     }else{
         printf("testqset_r: OK\n\n");
     }
@@ -239,12 +245,12 @@ void readOptions(qhT *qh, int argc, char **argv, const char *promptstr, int *num
     }
     numIntsArg= strtol(argv[1], &endp, 10);
     if(numIntsArg<1){
-        qh_fprintf(qh, stderr, 6301, "First argument should be 1 or greater.  Got '%s'\n", argv[1]);
-        exit(1);
+        qh_fprintf(qh, stderr, 6301, "testqset_r: First argument should be 1 or greater.  Got '%s'\n", argv[1]);
+        exit(qh_ERRinput);
     }
     if(numIntsArg>MAXint){
-        qh_fprintf(qh, stderr, 6302, "qset does not currently support 64-bit ints.  Maximum count is %d\n", MAXint);
-        exit(1);
+        qh_fprintf(qh, stderr, 6302, "testqset_r: qset does not currently support 64-bit ints.  Maximum count is %d\n", MAXint);
+        exit(qh_ERRinput);
     }
     *numInts= (int)numIntsArg;
 
@@ -255,20 +261,20 @@ void readOptions(qhT *qh, int argc, char **argv, const char *promptstr, int *num
     if(argc==4 || (argc==3 && !isTracing)){
         checkEveryArg= strtol(argv[2], &endp, 10);
         if(checkEveryArg<1){
-            qh_fprintf(qh, stderr, 6321, "checkEvery argument should be 1 or greater.  Got '%s'\n", argv[2]);
-            exit(1);
+            qh_fprintf(qh, stderr, 6321, "testqset_r: checkEvery argument should be 1 or greater.  Got '%s'\n", argv[2]);
+            exit(qh_ERRinput);
         }
         if(checkEveryArg>MAXint){
-            qh_fprintf(qh, stderr, 6322, "qset does not currently support 64-bit ints.  Maximum checkEvery is %d\n", MAXint);
-            exit(1);
+            qh_fprintf(qh, stderr, 6322, "testqset_r: qset does not currently support 64-bit ints.  Maximum checkEvery is %d\n", MAXint);
+            exit(qh_ERRinput);
         }
         if(argc==4){
             if(argv[3][0]=='T' && argv[3][1]=='5' ){
                 isTracing= 1;
                 *traceLevel= 5;
             }else{
-                qh_fprintf(qh, stderr, 6242, "Optional third argument must be 'T5'.  Got '%s'\n", argv[3]);
-                exit(1);
+                qh_fprintf(qh, stderr, 6242, "testqset_r: Optional third argument must be 'T5'.  Got '%s'\n", argv[3]);
+                exit(qh_ERRinput);
             }
         }
         *checkEvery= (int)checkEveryArg;
@@ -279,13 +285,13 @@ void setupMemory(qhT *qh, int tracelevel, int numInts, int **intarray)
 {
     int i;
     if(numInts<0 || numInts*(int)sizeof(int)<0){
-        qh_fprintf(qh, stderr, 6303, "qset does not currently support 64-bit ints.  Integer overflow\n");
-        exit(1);
+        qh_fprintf(qh, stderr, 6303, "testqset_r: qset does not currently support 64-bit ints.  Integer overflow\n");
+        exit(qh_ERRinput);
     }
-    *intarray= qh_malloc(numInts * sizeof(int));
+    *intarray= (int *)qh_malloc((unsigned int)numInts * sizeof(int));
     if(!*intarray){
-        qh_fprintf(qh, stderr, 6304, "Failed to allocate %d bytes of memory\n", numInts * sizeof(int));
-        exit(1);
+        qh_fprintf(qh, stderr, 6304, "testqset_r: Failed to allocate %d bytes of memory\n", numInts * (int)sizeof(int));
+        exit(qh_ERRmem);
     }
     for(i= 0; i<numInts; i++){
         (*intarray)[i] =i;
@@ -299,7 +305,7 @@ void setupMemory(qhT *qh, int tracelevel, int numInts, int **intarray)
     qh_memsize(qh, 40);
     qh_memsetup(qh);
 
-    qh_fprintf(qh, stderr, 8001, "SETelemsize is %d bytes for pointer-to-int\n", SETelemsize);
+    qh_fprintf(qh, stderr, 8089, "SETelemsize is %d bytes for pointer-to-int\n", SETelemsize);
 }/*setupMemmory*/
 
 void testSetappendSettruncate(qhT *qh, int numInts, int *intarray, int checkEvery)
@@ -307,7 +313,7 @@ void testSetappendSettruncate(qhT *qh, int numInts, int *intarray, int checkEver
     setT *ints= qh_setnew(qh, 4);
     int i, isCheck;
 
-    qh_fprintf(qh, stderr, 8002, "\n\nTesting qh_setappend 0..%d.  Test", numInts-1);
+    qh_fprintf(qh, stderr, 8108, "\n\nTesting qh_setappend 0..%d.  Test", numInts-1);
     for(i= 0; i<numInts; i++){
         isCheck= log_i(qh, ints, "i", i, numInts, checkEvery);
         qh_setappend(qh, &ints, intarray+i);
@@ -316,7 +322,7 @@ void testSetappendSettruncate(qhT *qh, int numInts, int *intarray, int checkEver
         }
     }
 
-    qh_fprintf(qh, stderr, 8014, "\n\nTesting qh_settruncate %d and 0.  Test", numInts/2);
+    qh_fprintf(qh, stderr, 8120, "\n\nTesting qh_settruncate %d and 0.  Test", numInts/2);
     if(numInts>=2){
         isCheck= log_i(qh, ints, "n", numInts/2, numInts, checkEvery);
         qh_settruncate(qh, ints, numInts/2);
@@ -326,7 +332,7 @@ void testSetappendSettruncate(qhT *qh, int numInts, int *intarray, int checkEver
     qh_settruncate(qh, ints, 0);
     checkSetContents(qh, "qh_settruncate", ints, 0, -1, -1, -1);
 
-    qh_fprintf(qh, stderr, 8003, "\n\nTesting qh_setappend2ndlast 0,0..%d.  Test 0", numInts-1);
+    qh_fprintf(qh, stderr, 8122, "\n\nTesting qh_setappend2ndlast 0,0..%d.  Test 0", numInts-1);
     qh_setfree(qh, &ints);
     ints= qh_setnew(qh, 4);
     qh_setappend(qh, &ints, intarray+0);
@@ -337,7 +343,7 @@ void testSetappendSettruncate(qhT *qh, int numInts, int *intarray, int checkEver
             checkSetContents(qh, "qh_setappend2ndlast", ints, i+2, 0, 0, -1);
         }
     }
-    qh_fprintf(qh, stderr, 8015, "\n\nTesting SETtruncate_ %d and 0.  Test", numInts/2);
+    qh_fprintf(qh, stderr, 8146, "\n\nTesting SETtruncate_ %d and 0.  Test", numInts/2);
     if(numInts>=2){
         isCheck= log_i(qh, ints, "n", numInts/2, numInts, checkEvery);
         SETtruncate_(ints, numInts/2);
@@ -355,7 +361,7 @@ void testSetdelSetadd(qhT *qh, int numInts, int *intarray, int checkEvery)
     setT *ints=qh_setnew(qh, 1);
     int i,j,isCheck;
 
-    qh_fprintf(qh, stderr, 8003, "\n\nTesting qh_setdelnthsorted and qh_setaddnth 1..%d. Test", numInts-1);
+    qh_fprintf(qh, stderr, 8147, "\n\nTesting qh_setdelnthsorted and qh_setaddnth 1..%d. Test", numInts-1);
     for(j=1; j<numInts; j++){  /* size 0 not valid */
         if(log_i(qh, ints, "j", j, numInts, MAXint)){
             for(i= qh_setsize(qh, ints); i<j; i++){
@@ -383,7 +389,7 @@ void testSetappendSet(qhT *qh, int numInts, int *intarray, int checkEvery)
     setT *ints2;
     int i,j,k;
 
-    qh_fprintf(qh, stderr, 8016, "\n\nTesting qh_setappend_set 0..%d. Test", numInts-1);
+    qh_fprintf(qh, stderr, 8148, "\n\nTesting qh_setappend_set 0..%d. Test", numInts-1);
     for(j=0; j<numInts; j++){
         if(log_i(qh, ints, "j", j, numInts, numInts)){
             for(i= qh_setsize(qh, ints); i<j; i++){
@@ -421,7 +427,7 @@ void testSetcompactCopy(qhT *qh, int numInts, int *intarray, int checkEvery)
     setT *ints2= NULL;
     int i,j,k;
 
-    qh_fprintf(qh, stderr, 8017, "\n\nTesting qh_setcompact and qh_setcopy 0..%d. Test", numInts-1);
+    qh_fprintf(qh, stderr, 8149, "\n\nTesting qh_setcompact and qh_setcopy 0..%d. Test", numInts-1);
     for(j=0; j<numInts; j++){
         if(log_i(qh, ints, "j", j, numInts, checkEvery)){
             for(i= qh_setsize(qh, ints); i<j; i++){  /* Test i<j to test the empty set */
@@ -450,7 +456,7 @@ void testSetdelsortedEtc(qhT *qh, int numInts, int *intarray, int checkEvery)
     setT *ints2= NULL;
     int i,j;
 
-    qh_fprintf(qh, stderr, 8018, "\n\nTesting qh_setdel*, qh_setaddsorted, and  0..%d. Test", numInts-1);
+    qh_fprintf(qh, stderr, 8150, "\n\nTesting qh_setdel*, qh_setaddsorted, and  0..%d. Test", numInts-1);
     for(j=0; j<numInts; j++){
         if(log_i(qh, ints, "j", j, numInts, checkEvery)){
             for(i= qh_setsize(qh, ints); i<j; i++){  /* Test i<j to test the empty set */
@@ -506,7 +512,7 @@ void testSetequalInEtc(qhT *qh, int numInts, int *intarray, int checkEvery)
     setT *ints3= NULL;
     int i,j,n;
 
-    qh_fprintf(qh, stderr, 8019, "\n\nTesting qh_setequal*, qh_setin*, qh_setdel, qh_setdelnth, and qh_setlarger 0..%d. Test", numInts-1);
+    qh_fprintf(qh, stderr, 8151, "\n\nTesting qh_setequal*, qh_setin*, qh_setdel, qh_setdelnth, and qh_setlarger 0..%d. Test", numInts-1);
     for(j=0; j<numInts; j++){
         if(log_i(qh, ints, "j", j, numInts, checkEvery)){
             n= qh_setsize(qh, ints);
@@ -517,16 +523,16 @@ void testSetequalInEtc(qhT *qh, int numInts, int *intarray, int checkEvery)
             }
             checkSetContents(qh, "qh_setappend", ints, j, 0, -1, -1);
             if(!qh_setequal(ints, ints)){
-                qh_fprintf(qh, stderr, 6300, "testSetequalInEtc: set not equal to itself at length %d\n", j);
+                qh_fprintf(qh, stderr, 6300, "testqset_r (testSetequalInEtc): set not equal to itself at length %d\n", j);
                 error_count++;
             }
             if(j==0 && !qh_setequal(ints, ints2)){
-                qh_fprintf(qh, stderr, 6323, "testSetequalInEtc: empty set not equal to null set\n");
+                qh_fprintf(qh, stderr, 6323, "testqset_r (testSetequalInEtc): empty set not equal to null set\n");
                 error_count++;
             }
             if(j>0){
                 if(qh_setequal(ints, ints2)){
-                    qh_fprintf(qh, stderr, 6324, "testSetequalInEtc: non-empty set equal to empty set\n", j);
+                    qh_fprintf(qh, stderr, 6324, "testqset_r (testSetequalInEtc): non-empty set equal to empty set\n", j);
                     error_count++;
                 }
                 qh_setfree(qh, &ints3);
@@ -541,27 +547,27 @@ void testSetequalInEtc(qhT *qh, int numInts, int *intarray, int checkEvery)
                     checkSetContents(qh, "qh_setreplace 3", ints3, j, 0, j/2+1, j/2+1);
                 }
                 if(qh_setequal(ints, ints3)){
-                    qh_fprintf(qh, stderr, 6325, "testSetequalInEtc: modified set equal to original set at %d/2\n", j);
+                    qh_fprintf(qh, stderr, 6325, "testqset_r (testSetequalInEtc): modified set equal to original set at %d/2\n", j);
                     error_count++;
                 }
                 if(!qh_setequal_except(ints, intarray+j/2, ints3, intarray+j/2+1)){
-                    qh_fprintf(qh, stderr, 6326, "qh_setequal_except: modified set not equal to original set except modified\n", j);
+                    qh_fprintf(qh, stderr, 6326, "testqset_r (qh_setequal_except): modified set not equal to original set except modified\n", j);
                     error_count++;
                 }
                 if(qh_setequal_except(ints, intarray+j/2, ints3, intarray)){
-                    qh_fprintf(qh, stderr, 6327, "qh_setequal_except: modified set equal to original set with wrong excepts\n", j);
+                    qh_fprintf(qh, stderr, 6327, "testqset_r (qh_setequal_except): modified set equal to original set with wrong excepts\n", j);
                     error_count++;
                 }
                 if(!qh_setequal_skip(ints, j/2, ints3, j/2)){
-                    qh_fprintf(qh, stderr, 6328, "qh_setequal_skip: modified set not equal to original set except modified\n", j);
+                    qh_fprintf(qh, stderr, 6328, "testqset_r (qh_setequal_skip): modified set not equal to original set except modified\n", j);
                     error_count++;
                 }
                 if(j>2 && qh_setequal_skip(ints, j/2, ints3, 0)){
-                    qh_fprintf(qh, stderr, 6329, "qh_setequal_skip: modified set equal to original set with wrong excepts\n", j);
+                    qh_fprintf(qh, stderr, 6329, "testqset_r (qh_setequal_skip): modified set equal to original set with wrong excepts\n", j);
                     error_count++;
                 }
                 if(intarray+j/2+1!=qh_setdel(ints3, intarray+j/2+1)){
-                    qh_fprintf(qh, stderr, 6330, "qh_setdel: failed to find added element\n", j);
+                    qh_fprintf(qh, stderr, 6330, "testqset_r (qh_setdel): failed to find added element\n", j);
                     error_count++;
                 }
                 checkSetContents(qh, "qh_setdel", ints3, j-1, 0, j-1, (j==1 ? -1 : j/2+1));  /* swaps last element with deleted element */
@@ -570,31 +576,31 @@ void testSetequalInEtc(qhT *qh, int numInts, int *intarray, int checkEvery)
                     checkSetContents(qh, "qh_setdelnth", ints3, j-2, 0, j-2, (j==2 ? -1 : j/2+1));
                 }
                 if(qh_setin(ints3, intarray+j/2)){
-                    qh_fprintf(qh, stderr, 6331, "qh_setin: found deleted element\n");
+                    qh_fprintf(qh, stderr, 6331, "testqset_r (qh_setin): found deleted element\n");
                     error_count++;
                 }
                 if(j>4 && !qh_setin(ints3, intarray+1)){
-                    qh_fprintf(qh, stderr, 6332, "qh_setin: did not find second element\n");
+                    qh_fprintf(qh, stderr, 6332, "testqset_r (qh_setin): did not find second element\n");
                     error_count++;
                 }
                 if(j>4 && !qh_setin(ints3, intarray+j-2)){
-                    qh_fprintf(qh, stderr, 6333, "qh_setin: did not find last element\n");
+                    qh_fprintf(qh, stderr, 6333, "testqset_r (qh_setin): did not find last element\n");
                     error_count++;
                 }
                 if(-1!=qh_setindex(ints2, intarray)){
-                    qh_fprintf(qh, stderr, 6334, "qh_setindex: found element in empty set\n");
+                    qh_fprintf(qh, stderr, 6334, "testqset_r (qh_setindex): found element in empty set\n");
                     error_count++;
                 }
                 if(-1!=qh_setindex(ints3, intarray+j/2)){
-                    qh_fprintf(qh, stderr, 6335, "qh_setindex: found deleted element in set\n");
+                    qh_fprintf(qh, stderr, 6335, "testqset_r (qh_setindex): found deleted element in set\n");
                     error_count++;
                 }
                 if(0!=qh_setindex(ints, intarray)){
-                    qh_fprintf(qh, stderr, 6336, "qh_setindex: did not find first in set\n");
+                    qh_fprintf(qh, stderr, 6336, "testqset_r (qh_setindex): did not find first in set\n");
                     error_count++;
                 }
                 if(j-1!=qh_setindex(ints, intarray+j-1)){
-                    qh_fprintf(qh, stderr, 6337, "qh_setindex: did not find last in set\n");
+                    qh_fprintf(qh, stderr, 6337, "testqset_r (qh_setindex): did not find last in set\n");
                     error_count++;
                 }
             }
@@ -615,40 +621,40 @@ void testSetlastEtc(qhT *qh, int numInts, int *intarray, int checkEvery)
     setT *ints2= NULL;
     int i,j,prepend;
 
-    qh_fprintf(qh, stderr, 8020, "\n\nTesting qh_setlast, qh_setnew_delnthsorted, qh_setunique, and qh_setzero 0..%d. Test", numInts-1);
+    qh_fprintf(qh, stderr, 8152, "\n\nTesting qh_setlast, qh_setnew_delnthsorted, qh_setunique, and qh_setzero 0..%d. Test", numInts-1);
     for(j=0; j<numInts; j++){
         if(log_i(qh, ints, "j", j, numInts, checkEvery)){
             for(i= qh_setsize(qh, ints); i<j; i++){  /* Test i<j to test the empty set */
                 if(!qh_setunique(qh, &ints, intarray+i)){
-                    qh_fprintf(qh, stderr, 6340, "qh_setunique: not able to append next element %d\n", i);
+                    qh_fprintf(qh, stderr, 6340, "testqset_r (qh_setunique): not able to append next element %d\n", i);
                     error_count++;
                 }
                 if(checkEvery==1){
                     checkSetContents(qh, "qh_setunique", ints, i+1, 0, -1, -1);
                 }
                 if(qh_setunique(qh, &ints, intarray+i)){
-                    qh_fprintf(qh, stderr, 6341, "qh_setunique: appended next element twice %d\n", i);
+                    qh_fprintf(qh, stderr, 6341, "testqset_r (qh_setunique): appended next element twice %d\n", i);
                     error_count++;
                 }
                 if(qh_setunique(qh, &ints, intarray+i/2)){
-                    qh_fprintf(qh, stderr, 6346, "qh_setunique: appended middle element twice %d/2\n", i);
+                    qh_fprintf(qh, stderr, 6346, "testqset_r (qh_setunique): appended middle element twice %d/2\n", i);
                     error_count++;
                 }
             }
             checkSetContents(qh, "qh_setunique 2", ints, j, 0, -1, -1);
             if(j==0 && NULL!=qh_setlast(ints)){
-                qh_fprintf(qh, stderr, 6339, "qh_setlast: returned last element of empty set\n");
+                qh_fprintf(qh, stderr, 6339, "testqset_r (qh_setlast): returned last element of empty set\n");
                 error_count++;
             }
             if(j>0){
                 if(intarray+j-1!=qh_setlast(ints)){
-                    qh_fprintf(qh, stderr, 6338, "qh_setlast: wrong last element\n");
+                    qh_fprintf(qh, stderr, 6338, "testqset_r (qh_setlast): wrong last element\n");
                     error_count++;
                 }
                 prepend= (j<100 ? j/4 : 0);
                 ints2= qh_setnew_delnthsorted(qh, ints, qh_setsize(qh, ints), j/2, prepend);
                 if(qh_setsize(qh, ints2)!=j+prepend-1){
-                    qh_fprintf(qh, stderr, 6345, "qh_setnew_delnthsorted: Expecting %d elements, got %d\n", j+prepend-1, qh_setsize(qh, ints2));
+                    qh_fprintf(qh, stderr, 6345, "testqset_r (qh_setnew_delnthsorted): Expecting %d elements, got %d\n", j+prepend-1, qh_setsize(qh, ints2));
                     error_count++;
                 }
                 /* Define prepended elements.  Otherwise qh_setdelnthsorted may fail */
@@ -663,7 +669,7 @@ void testSetlastEtc(qhT *qh, int numInts, int *intarray, int checkEvery)
                 if(j>2){
                     qh_setzero(qh, ints2, j/2, j-1);  /* max size may be j-1 */
                     if(qh_setsize(qh, ints2)!=j-1){
-                        qh_fprintf(qh, stderr, 6342, "qh_setzero: Expecting %d elements, got %d\n", j, qh_setsize(qh, ints2));
+                        qh_fprintf(qh, stderr, 6342, "testqset_r (qh_setzero): Expecting %d elements, got %d\n", j, qh_setsize(qh, ints2));
                         error_count++;
                     }
                     qh_setcompact(qh, ints2);
@@ -686,7 +692,7 @@ void testSettemp(qhT *qh, int numInts, int *intarray, int checkEvery)
     setT *ints3= NULL;
     int i,j;
 
-    qh_fprintf(qh, stderr, 8021, "\n\nTesting qh_settemp* 0..%d. Test", numInts-1);
+    qh_fprintf(qh, stderr, 8153, "\n\nTesting qh_settemp* 0..%d. Test", numInts-1);
     for(j=0; j<numInts; j++){
         if(log_i(qh, ints, "j", j, numInts, checkEvery)){
             if(j<20){
@@ -703,7 +709,7 @@ void testSettemp(qhT *qh, int numInts, int *intarray, int checkEvery)
                 qh_settemppush(qh, ints);
                 ints3= qh_settemppop(qh);
                 if(ints!=ints3){
-                    qh_fprintf(qh, stderr, 6343, "qh_settemppop: didn't pop the push\n");
+                    qh_fprintf(qh, stderr, 6343, "testqset_r (qh_settemppop): didn't pop the push\n");
                     error_count++;
                 }
             }
@@ -730,26 +736,26 @@ int log_i(qhT *qh, setT *set, const char *s, int i, int numInts, int checkEvery)
 
     if(*s || checkEvery==1){
         if(i<10){
-            qh_fprintf(qh, stderr, 8004, " %s%d", s, i);
+            qh_fprintf(qh, stderr, 8154, " %s%d", s, i);
         }else{
             if(i==11 && checkEvery==1){
-                qh_fprintf(qh, stderr, 8005, "\nResults after 10: ");
+                qh_fprintf(qh, stderr, 8155, "\nResults after 10: ");
                 FOREACHint_(set){
-                    qh_fprintf(qh, stderr, 8006, " %d", *i2);
+                    qh_fprintf(qh, stderr, 8156, " %d", *i2);
                 }
-                qh_fprintf(qh, stderr, 8007, " Continue");
+                qh_fprintf(qh, stderr, 8157, " Continue");
             }
             while((j= j/10)>=1){
                 scale *= 10;
                 e++;
             }
             if(i==numInts-1){
-                qh_fprintf(qh, stderr, 8008, " %s%d", s, i);
+                qh_fprintf(qh, stderr, 8158, " %s%d", s, i);
             }else if(i==scale){
                 if(i<=1000){
-                    qh_fprintf(qh, stderr, 8010, " %s%d", s, i);
+                    qh_fprintf(qh, stderr, 8159, " %s%d", s, i);
                 }else{
-                    qh_fprintf(qh, stderr, 8009, " %s1e%d", s, e);
+                    qh_fprintf(qh, stderr, 8160, " %s1e%d", s, e);
                 }
             }
         }
@@ -780,38 +786,38 @@ void checkSetContents(qhT *qh, const char *name, setT *set, int count, int range
     if(set){
         SETreturnsize_(set, actualSize);  /* normally used only when speed is critical */
         if(*qh_setendpointer(set)!=NULL){
-            qh_fprintf(qh, stderr, 6344, "%s: qh_setendpointer(set), 0x%x, is not NULL terminator of set 0x%x", name, qh_setendpointer(set), set);
+            qh_fprintf(qh, stderr, 6344, "testqset_r (%s): qh_setendpointer(set), 0x%x, is not NULL terminator of set 0x%x\n", name, qh_setendpointer(set), set);
             error_count++;
         }
     }
     if(actualSize!=qh_setsize(qh, set)){
-        qh_fprintf(qh, stderr, 6305, "%s: SETreturnsize_(qh) returned %d while qh_setsize(qh) returns %d\n", name, actualSize, qh_setsize(qh, set));
+        qh_fprintf(qh, stderr, 6305, "testqset_r (%s): SETreturnsize_(qh) returned %d while qh_setsize(qh) returns %d\n", name, actualSize, qh_setsize(qh, set));
         error_count++;
     }else if(actualSize!=count){
-        qh_fprintf(qh, stderr, 6306, "%s: Expecting %d elements for set.  Got %d elements\n", name, count, actualSize);
+        qh_fprintf(qh, stderr, 6306, "testqset_r (%s): Expecting %d elements for set.  Got %d elements\n", name, count, actualSize);
         error_count++;
     }
     if(SETempty_(set)){
         if(count!=0){
-            qh_fprintf(qh, stderr, 6307, "%s: Got empty set instead of count %d, rangeA %d, rangeB %d, rangeC %d\n", name, count, rangeA, rangeB, rangeC);
+            qh_fprintf(qh, stderr, 6307, "testqset_r (%s): Got empty set instead of count %d, rangeA %d, rangeB %d, rangeC %d\n", name, count, rangeA, rangeB, rangeC);
             error_count++;
         }
     }else{
         /* Must be first, otherwise trips msvc 8 */
         i2T **p= SETaddr_(set, i2T);
         if(*p!=SETfirstt_(set, i2T)){
-            qh_fprintf(qh, stderr, 6309, "%s: SETaddr_(set, i2t) [%p] is not the same as SETfirst_(set) [%p]\n", name, SETaddr_(set, i2T), SETfirst_(set));
+            qh_fprintf(qh, stderr, 6309, "testqset_r (%s): SETaddr_(set, i2t) [%p] is not the same as SETfirst_(set) [%p]\n", name, SETaddr_(set, i2T), SETfirst_(set));
             error_count++;
         }
         first= *(int *)SETfirst_(set);
         if(SETfirst_(set)!=SETfirstt_(set, i2T)){
-            qh_fprintf(qh, stderr, 6308, "%s: SETfirst_(set) [%p] is not the same as SETfirstt_(set, i2T [%p]\n", name, SETfirst_(set), SETfirstt_(set, i2T));
+            qh_fprintf(qh, stderr, 6308, "testqset_r (%s): SETfirst_(set) [%p] is not the same as SETfirstt_(set, i2T [%p]\n", name, SETfirst_(set), SETfirstt_(set, i2T));
             error_count++;
         }
         if(qh_setsize(qh, set)>1){
             second= *(int *)SETsecond_(set);
             if(SETsecond_(set)!=SETsecondt_(set, i2T)){
-                qh_fprintf(qh, stderr, 6310, "%s: SETsecond_(set) [%p] is not the same as SETsecondt_(set, i2T) [%p]\n", name, SETsecond_(set), SETsecondt_(set, i2T));
+                qh_fprintf(qh, stderr, 6310, "testqset_r (%s): SETsecond_(set) [%p] is not the same as SETsecondt_(set, i2T) [%p]\n", name, SETsecond_(set), SETsecondt_(set, i2T));
                 error_count++;
             }
         }
@@ -824,11 +830,11 @@ void checkSetContents(qhT *qh, const char *name, setT *set, int count, int range
         }
         prev= *i2;
         if(SETindex_(set, i2)!=i){
-            qh_fprintf(qh, stderr, 6311, "%s: Expecting SETindex_(set, pointer-to-%d) to be %d.  Got %d\n", name, *i2, i, SETindex_(set, i2));
+            qh_fprintf(qh, stderr, 6311, "testqset_r (%s): Expecting SETindex_(set, pointer-to-%d) to be %d.  Got %d\n", name, *i2, i, SETindex_(set, i2));
             error_count++;;
         }
         if(i2!=SETref_(i2)){
-            qh_fprintf(qh, stderr, 6312, "%s: SETref_(i2) [%p] does not point to i2 (the %d'th element)\n", name, SETref_(i2), i);
+            qh_fprintf(qh, stderr, 6312, "testqset_r (%s): SETref_(i2) [%p] does not point to i2 (the %d'th element)\n", name, SETref_(i2), i);
             error_count++;;
         }
         i++;
@@ -837,22 +843,22 @@ void checkSetContents(qhT *qh, const char *name, setT *set, int count, int range
         /* Must be first conditional, otherwise it trips up msvc 8 */
         i2T **p= SETelemaddr_(set, i2_i, i2T);
         if(i2!=*p){
-            qh_fprintf(qh, stderr, 6320, "%s: SETelemaddr_(set, %d, i2T) [%p] does not point to i2\n", name, i2_i, SETelemaddr_(set, i2_i, int));
+            qh_fprintf(qh, stderr, 6320, "testqset_r (%s): SETelemaddr_(set, %d, i2T) [%p] does not point to i2\n", name, i2_i, SETelemaddr_(set, i2_i, int));
             error_count++;;
         }
         if(i2_i==0){
             if(first!=*i2){
-                qh_fprintf(qh, stderr, 6314, "%s: First element is %d instead of SETfirst %d\n", name, *i2, first);
+                qh_fprintf(qh, stderr, 6314, "testqset_r (%s): First element is %d instead of SETfirst %d\n", name, *i2, first);
                 error_count++;;
             }
             if(rangeA!=*i2){
-                qh_fprintf(qh, stderr, 6315, "%s: starts with %d instead of rangeA %d\n", name, *i2, rangeA);
+                qh_fprintf(qh, stderr, 6315, "testqset_r (%s): starts with %d instead of rangeA %d\n", name, *i2, rangeA);
                 error_count++;;
             }
             prev= rangeA;
         }else{
             if(i2_i==1 && second!=*i2){
-                qh_fprintf(qh, stderr, 6316, "%s: Second element is %d instead of SETsecond %d\n", name, *i2, second);
+                qh_fprintf(qh, stderr, 6316, "testqset_r (%s): Second element is %d instead of SETsecond %d\n", name, *i2, second);
                 error_count++;;
             }
             if(prev+1==*i2){
@@ -868,22 +874,22 @@ void checkSetContents(qhT *qh, const char *name, setT *set, int count, int range
                     rangeCount++;
                 }else{
                     prev++;
-                    qh_fprintf(qh, stderr, 6317, "%s: Expecting %d'th element to be %d.  Got %d\n", name, i2_i, prev, *i2);
+                    qh_fprintf(qh, stderr, 6317, "testqset_r (%s): Expecting %d'th element to be %d.  Got %d\n", name, i2_i, prev, *i2);
                     error_count++;
                 }
             }
         }
         if(i2!=SETelem_(set, i2_i)){
-            qh_fprintf(qh, stderr, 6318, "%s: SETelem_(set, %d) [%p] is not i2 [%p] (the %d'th element)\n", name, i2_i, SETelem_(set, i2_i), i2, i2_i);
+            qh_fprintf(qh, stderr, 6318, "testqset_r (%s): SETelem_(set, %d) [%p] is not i2 [%p] (the %d'th element)\n", name, i2_i, SETelem_(set, i2_i), i2, i2_i);
             error_count++;;
         }
         if(SETelemt_(set, i2_i, i2T)!=SETelem_(set, i2_i)){   /* Normally SETelemt_ is used for generic sets */
-            qh_fprintf(qh, stderr, 6319, "%s: SETelemt_(set, %d, i2T) [%p] is not SETelem_(set, %d) [%p] (the %d'th element)\n", name, i2_i, SETelemt_(set, i2_i, int), i2_i, SETelem_(set, i2_i), i2_i);
+            qh_fprintf(qh, stderr, 6319, "testqset_r (%s): SETelemt_(set, %d, i2T) [%p] is not SETelem_(set, %d) [%p] (the %d'th element)\n", name, i2_i, SETelemt_(set, i2_i, int), i2_i, SETelem_(set, i2_i), i2_i);
             error_count++;;
         }
     }
     if(error_count>=MAXerrorCount){
-        qh_fprintf(qh, stderr, 8011, "testqset: Stop testing after %d errors\n", error_count);
+        qh_fprintf(qh, stderr, 8161, "testqset_r (checkSetContents): Stop testing after %d errors\n", error_count);
         exit(1);
     }
 }/*checkSetContents*/
